import { DenoBridge } from "@netlify/edge-bundler";
import { join, relative, dirname } from "path";
import { readFile, stat, mkdir, writeFile } from "fs/promises";
import * as JSONC from "comment-json";
import { confirm } from "@inquirer/prompts";
import unixify from "unixify";
import open from "open";
import { execSync } from "child_process";
export const PACKAGE_MANAGERS = ["npm", "yarn", "pnpm"];
export const isPackageManager = (value) => {
    return PACKAGE_MANAGERS.includes(value);
};
const PACKAGE_MANAGER_COMMANDS = {
    install: {
        npm: "npm install",
        yarn: "yarn",
        pnpm: "pnpm install",
    },
    add: {
        npm: "npm install",
        yarn: "yarn add",
        pnpm: "pnpm add",
    },
    run: {
        npm: "npx",
        yarn: "yarn",
        pnpm: "pnpm",
    },
};
export const getCommand = (command, packageManager, args) => {
    const pkgManagerCommand = PACKAGE_MANAGER_COMMANDS[command][packageManager];
    if (!args) {
        return pkgManagerCommand;
    }
    return `${pkgManagerCommand} ${args.join(" ")}`;
};
export const getSettings = async (settingsPath) => {
    try {
        const stats = await stat(settingsPath);
        if (!stats.isFile()) {
            throw new Error(`${settingsPath} is not a valid file.`);
        }
        const file = await readFile(settingsPath, "utf8");
        return {
            fileExists: true,
            settings: JSONC.parse(file),
        };
    }
    catch (error) {
        if (error.code !== "ENOENT") {
            throw new Error(`Could not open VS Code settings file: ${error.message}`);
        }
        return {
            fileExists: false,
            settings: {},
        };
    }
};
export const applySettings = (existingSettings, { denoBinary, edgeFunctionsPath, destination, }) => {
    const relativeEdgeFunctionsPath = unixify(relative(destination, edgeFunctionsPath));
    const settings = JSONC.assign(existingSettings, {
        "deno.enable": true,
        "deno.enablePaths": existingSettings["deno.enablePaths"] || [],
        "deno.unstable": true,
    });
    // If the Edge Functions path isn't already in `deno.enabledPaths`, let's add
    // it.
    if (!settings["deno.enablePaths"].includes(relativeEdgeFunctionsPath)) {
        settings["deno.enablePaths"].push(relativeEdgeFunctionsPath);
    }
    // If the Deno CLI binary isn't globally installed, we need to set the path
    // to it in the settings file or the extension won't know where to find it.
    // The only exception is when `deno.path` has already been defined, because
    // we don't want to override that.
    if (!denoBinary.global && settings["deno.path"] === undefined) {
        settings["deno.path"] = denoBinary.path;
    }
    return settings;
};
export const writeSettings = async ({ settings, settingsPath, }) => {
    const serializedSettings = JSONC.stringify(settings, null, 2);
    await mkdir(dirname(settingsPath), { recursive: true });
    await writeFile(settingsPath, serializedSettings);
};
export const promptEditorHelper = async ({ destination, }) => {
    const deno = new DenoBridge({
        onBeforeDownload: () => console.log(`Setting up the Edge Functions environment. This may take a couple of minutes.`),
    });
    const denoBinary = await deno.getBinaryPath();
    const settingsPath = join(destination, ".vscode", "settings.json");
    const edgeFunctionsPath = join(destination, "src", "edge-functions");
    const { fileExists, settings: existingSettings } = await getSettings(settingsPath);
    const settings = applySettings(existingSettings, {
        denoBinary,
        edgeFunctionsPath,
        destination,
    });
    const createOrUpdateSettings = await confirm({
        message: fileExists
            ? `There is a VS Code settings file at ${settingsPath}. Can we update it?`
            : `A new VS Code settings file will be created at ${settingsPath}`,
        default: true,
    });
    if (!createOrUpdateSettings) {
        return;
    }
    try {
        const denoExtConfirm = await confirm({
            message: "The Deno VS Code extension is recommended. Would you like to install it now?",
            default: true,
        });
        if (denoExtConfirm) {
            console.log("Opening your browser...");
            open("https://ntl.fyi/deno-vscode");
        }
    }
    catch {
        console.log("Something went wrong opening the browser, install the deno vs-code extension manually here: https://ntl.fyi/deno-vscode");
    }
    try {
        await writeSettings({ settings, settingsPath });
        console.log(`VS Code settings file ${fileExists ? "updated" : "created"}.`);
    }
    catch {
        throw new Error("Could not write VS Code settings file.");
    }
};
export const isNetlifyCLIInstalled = () => {
    try {
        execSync("netlify --version", { stdio: "ignore" });
        return true;
    }
    catch {
        return false;
    }
};
//# sourceMappingURL=utils.js.map